import chartUtils from '../../utils/chart-utils.js';

/*--------------------------------外部调用----------------------------*/

/* 封装 - 在容器中添加图层 */
const addLayer = function(container, layerInfo) {
  logger.debug(`图表调试 - ${layerInfo.name} - addLayer`, container, layerInfo);
  createLayer(container, layerInfo);
};

/* 封装 - 在容器中移除图层 */
const removeLayer = function(container, layerInfo) {
  logger.debug(
    `图表调试 - ${layerInfo.name} - removeLayer`,
    container,
    layerInfo
  );
  deleteLayer(container, layerInfo);
};

/* 封装 - 在容器中显隐改变 */
const visibleChange = function(container, layerInfo, isShow) {
  logger.debug(
    `图表调试 - ${layerInfo.name} - visibleChange`,
    container,
    layerInfo,
    isShow
  );
  layerVisible(container, layerInfo, isShow);
};

/* 封装 - 在容器中样式改变 */
const styleChange = function(container, layerInfo, newStyle) {
  logger.debug(
    `图表调试 - ${layerInfo.name} - styleChange`,
    container,
    layerInfo,
    newStyle
  );

  let styles = layerInfo.component.style;
  let oldStyle = styles.find(
    (s) => s.name.toLowerCase() == newStyle.name.toLowerCase()
  );
  if (oldStyle) {
    oldStyle.value = newStyle.value;
    refreshLayer(container, layerInfo);
  }
};

/* 封装 - 在容器中数据改变 */
const dataChange = function(container, layerInfo, newData) {
  logger.debug(
    `图表调试 - ${layerInfo.name} - dataChange`,
    container,
    layerInfo,
    newData
  );
};

/*--------------------------------图层操作----------------------------*/

/* 创建图层 */
const createLayer = function(container, layerInfo) {
  let layerName = layerInfo.id;
  let points = getData(layerInfo);
  let style = getStyle(layerInfo);

  for (let point of points) {
    let canvas = createCanvas(layerInfo, point.label);
    addPoint(container, layerName, point, style, canvas);
    addPanel(container, layerInfo, point);
  }
};

/* 刷新图层 */
const refreshLayer = function(container, layerInfo) {
  let layerName = layerInfo.id;
  let points = getData(layerInfo);
  let style = getStyle(layerInfo);

  for (let point of points) {
    container.removeLandmark(layerName, point.batch);
    let canvas = createCanvas(layerInfo, point.label);
    addPoint(container, layerName, point, style, canvas);
    //addPanel(container, layerInfo, point);
  }
};

/* 添加地标点 */
const addPoint = function(container, layerName, point, style, canvas) {
  var pos = new avw.Position('Default');
  pos = pos.set(point.lon, point.lat, point.alt);

  var landMarkInfo = {
    type: 'landmark',
    name: point.batch,
    position: pos,
    icon: style.icon,
    width: 10,
    height: 10,
    iconXOffset: 0,
    iconYOffset: 0,

    scale: style.enableScaling,
    smallIconMax: style.smallIconMax,
    smallIconMin: style.smallIconMin,
    bigIconMax: style.bigIconMax,
    bigIconMin: style.bigIconMin,
    smallIconScale: style.smallIconFactor,
    bigIconScale: style.bigIconFactor,

    label: style.enableLabel,
    labelXOffset: style.labelHorOffset,
    labelYOffset: style.labelVerOffset,
    labelFontSize: canvas.fontSize,
    labelCanvas: canvas.canvas,

    offset: style.enableOffset,
    lonOffset: style.lonOffset,
    latOffset: style.latOffset,
    altOffset: style.altOffset,

    enableHeightMap: style.staickLand,
  };
  container.addLandmark(layerName, landMarkInfo);
};

/* 添加标牌 */
const addPanel = function(container, layerInfo, point) {
  let styles = layerInfo.component.style;

  let bgColor = chartUtils.getStyleValue(styles, 'tooltipBackgroundColor');
  let bdColor = chartUtils.getStyleValue(styles, 'tooltipBorderColor');
  let isShowShadow = chartUtils.getStyleValue(styles, 'tooltipIsShowShadow');
  let textSty = chartUtils.getStyleValue(styles, 'tooltipTextStyle');
  let styObj = chartUtils.getTextStyleObj(textSty.value[0].items);

  let fontStyle = `style="font-size: ${styObj.fontSize}; font-style: ${styObj.fontStyle}; color: ${styObj.color}; background-color: ${styObj.backgroundColor}; font-family: ${styObj.fontFamily}; font-weight: ${styObj.fontWeight};  margin: 5px;"`;

  let content = `<div style="background-color: ${bgColor.color}; border: 1px solid ${bdColor.color}; text-align: left;">`;
  content += `<span ${fontStyle}>经度：${point.lon}</span><br>`;
  content += `<span ${fontStyle}>维度${point.lat}</span><br>`;
  content += `<span ${fontStyle}>高度：${point.alt}</span><br>`;
  content += `<span ${fontStyle}>批号：${point.batch}</span><br>`;
  content += `<span ${fontStyle}>类别：${point.category}</span><br>`;
  content += `<span ${fontStyle}>标签：${point.label}</span></div>`;

  let horOffset = chartUtils.getStyleValue(styles, 'tooltipHorOffset');
  let verOffset = chartUtils.getStyleValue(styles, 'tooltipVerOffset');
  container.addInfoPanel({
    name: point.batch,
    type: 'infopanel',
    exclusive: false,
    content: content,
    position: new avw.Position('Default').set(0, 0, 0),
    offset: [horOffset, verOffset, 0],
    width: 200,
    height: 'auto',
  });
};

/* 删除图层 */
const deleteLayer = function(container, layerInfo) {
  let points = getData(layerInfo);
  for (let point of points) {
    container.removeLandmark(layerInfo.id, point.batch);
  }
};

/* 图层显隐 */
const layerVisible = function(container, layerInfo, isShow) {
  //layerInfo.show
  //todo:目前没有显隐控制处理
  logger.debug('图表操作---layerVisible', layerInfo.show);
  if (isShow == true && layerInfo.show) {
    addLayer(container, layerInfo);
  } else {
    removeLayer(container, layerInfo);
  }
};

/*--------------------------------业务逻辑----------------------------*/

/* 获取标签 */
const createCanvas = function(layerInfo, labelText) {
  let styles = layerInfo.component.style;
  let textSty = chartUtils.getStyleValue(styles, 'labelTextStyle');
  let styObj = chartUtils.getTextStyleObj(textSty.value[0].items);

  //创建canvas部分
  let _canvas = document.createElement('canvas');
  let _ctx = _canvas.getContext('2d');

  _ctx.font = `${styObj.fontStyle} ${styObj.fontWeight} ${styObj.fontSize}px ${styObj.fontFamily}`;
  _ctx.textAlign = 'center';
  _ctx.textBaseline = 'middle';

  _canvas.width = (_ctx.measureText(labelText).width + styObj.fontSize) * 2;
  _canvas.height = styObj.fontSize * 3.6;

  _ctx.fillStyle = styObj.backgroundColor;
  _ctx.fillRect(0, 0, _canvas.width, _canvas.height);

  _ctx.font = `${styObj.fontStyle} ${styObj.fontWeight} ${styObj.fontSize *
    2}px ${styObj.fontFamily}`;
  _ctx.fillStyle = styObj.color;
  _ctx.textAlign = 'center';
  _ctx.textBaseline = 'middle';
  _ctx.fillText(labelText, _canvas.width / 2, _canvas.height / 2);

  return {
    canvas: _canvas,
    fontSize: styObj.fontSize,
  };
};

/* 所有热图的数据 */
const chartsData = {};

/* 获取数据 */
const getData = function(layerInfo) {
  let chartData = chartsData[layerInfo.id];
  if (!chartData) {
    let fields = layerInfo.component.data.fields;
    let rows = null;
    if (layerInfo.component.data.category == '静态输入') {
      rows = layerInfo.component.data.static.rowData;
    } else {
      //API的静态数据
    }

    chartData = {
      rows: rows,
      fields: fields,
    };

    chartsData[layerInfo.id] = chartData;
  }

  let lonName = chartUtils.getFieldBindingName(chartData.fields, 'lon');
  let latName = chartUtils.getFieldBindingName(chartData.fields, 'lat');
  let altName = chartUtils.getFieldBindingName(chartData.fields, 'alt');
  let batchName = chartUtils.getFieldBindingName(chartData.fields, 'batch');
  let categoryName = chartUtils.getFieldBindingName(
    chartData.fields,
    'category'
  );
  let labelName = chartUtils.getFieldBindingName(chartData.fields, 'label');

  var points = [];
  for (let row of chartData.rows) {
    points.push({
      lon: row[lonName],
      lat: row[latName],
      alt: row[altName],
      batch: row[batchName],
      category: row[categoryName],
      label: row[labelName],
    });
  }

  return points;
};

/* 获取样式 */
const getStyle = function(layerInfo) {
  let styles = layerInfo.component.style;
  //图标
  let icon = chartUtils.getStyleValue(styles, 'globalStyleIcon');
  let brightness = chartUtils.getStyleValue(styles, 'globalStyleBrightness');
  //图标缩放
  let enableScaling = chartUtils.getStyleValue(
    styles,
    'globalStyleDistanceScaling'
  );
  let smallIconMax = chartUtils.getStyleValue(
    styles,
    'globalStyleDistanceScalingSmallMax'
  );
  let smallIconMin = chartUtils.getStyleValue(
    styles,
    'globalStyleDistanceScalingSmallMin'
  );
  let bigIconMax = chartUtils.getStyleValue(
    styles,
    'globalStyleDistanceScalingBiglMax'
  );
  let bigIconMin = chartUtils.getStyleValue(
    styles,
    'globalStyleDistanceScalingBigMin'
  );
  let smallIconFactor = chartUtils.getStyleValue(
    styles,
    'globalStyleDistanceScalingSmallFactor'
  );
  let bigIconFactor = chartUtils.getStyleValue(
    styles,
    'globalStyleDistanceScalingBigFactor'
  );
  //图标偏移
  let enableOffset = chartUtils.getStyleValue(styles, 'offsetSetting');
  let lonOffset = chartUtils.getStyleValue(styles, 'offsetSettingLonOffset');
  let latOffset = chartUtils.getStyleValue(styles, 'offsetSettingLatOffset');
  let altOffset = chartUtils.getStyleValue(styles, 'offsetSettingAltOffset');
  //标签偏移
  let enableLabel = chartUtils.getStyleValue(styles, 'label');
  let labelHorOffset = chartUtils.getStyleValue(styles, 'labelHorOffset');
  let labelVerOffset = chartUtils.getStyleValue(styles, 'labelVerOffset');
  //启用贴地
  let staickLand = chartUtils.getStyleValue(
    styles,
    'offsetSettingEnableStickLand'
  );

  return {
    icon: icon,
    brightness: brightness,
    enableScaling: enableScaling,
    smallIconMax: smallIconMax,
    smallIconMin: smallIconMin,
    bigIconMax: bigIconMax,
    bigIconMin: bigIconMin,
    smallIconFactor: smallIconFactor,
    bigIconFactor: bigIconFactor,
    enableOffset: enableOffset,
    lonOffset: lonOffset,
    latOffset: latOffset,
    altOffset: altOffset,
    enableLabel: enableLabel,
    labelHorOffset: labelHorOffset,
    labelVerOffset: labelVerOffset,
    staickLand: staickLand,
  };
};

export default {
  addLayer,
  removeLayer,
  visibleChange,
  styleChange,
  dataChange,
};
