import echarts from 'echarts';
import commonFunction from './common-functions';
import { createLogger } from 'vuex';

//#region 获取样式值

/* 获取样式值 */
const getStyleValue = function(styles, name, type1, type2) {
  var result = styles.find((s) => {
    return s.name && s.name.toLowerCase() == name.toLowerCase();
  });
  if (type1) {
    if (type1.toLowerCase() == 'color') {
      return getColor(result.value, type2);
    }
  }
  return result.value;
};

/* 将属性改为颜色值 */
const getColor = function(propertyValue, type) {
  let result = 'red';
  if (propertyValue == null) {
    result = 'red';
  }
  if (propertyValue.type == 'single') {
    result = colorRgba(propertyValue.color);
  } else if (propertyValue.type == 'gradient') {
    if (type) {
      if (type == 'background') {
        result = `linear-gradient(${propertyValue.direction}deg,${
          propertyValue.colors[0].color
        } ${propertyValue.colors[0].offset * 100}%,${
          propertyValue.colors[1].color
        } ${propertyValue.colors[1].offset * 100}%)`;
      } else {
        let newDirection = Math.floor(propertyValue.direction / 360 + 0.5);
        propertyValue.direction -= newDirection * 360;
        let point = getColorEndPoint(0, 0, 0, 1, propertyValue.direction);
        result = new echarts.graphic.LinearGradient(
          point.x1,
          point.y1,
          point.x2,
          point.y2,
          [
            {
              offset: propertyValue.colors[0].offset,
              color: propertyValue.colors[0].color,
            },
            {
              offset: propertyValue.colors[1].offset,
              color: propertyValue.colors[1].color,
            },
          ]
        );
      }
    } else {
      result = `linear-gradient(${propertyValue.direction}deg,${
        propertyValue.colors[0].color
      } ${propertyValue.colors[0].offset * 100}%,${
        propertyValue.colors[1].color
      } ${propertyValue.colors[1].offset * 100}%)`;
    }
  } else {
    result = 'red';
  }

  return result;
};

/* 将颜色转换成RGBA的形式 */
const colorRgba = function(color) {
  let val = null;
  if (color.indexOf('#') > -1) {
    if (color.length > 8) {
      let alpha = color.slice(7, 9);
      alpha = +(parseInt(alpha, 16) / 255).toFixed(2);
      val =
        'rgba(' +
        parseInt('0x' + color.slice(1, 3)) +
        ',' +
        parseInt('0x' + color.slice(3, 5)) +
        ',' +
        parseInt('0x' + color.slice(5, 7)) +
        ',' +
        alpha +
        ')';
    } else {
      val =
        'rgba(' +
        parseInt('0x' + color.slice(1, 3)) +
        ',' +
        parseInt('0x' + color.slice(3, 5)) +
        ',' +
        parseInt('0x' + color.slice(5, 7)) +
        ',' +
        1 +
        ')';
    }
  } else {
    val = color;
  }

  return val;
};

/* 获取颜色结束点坐标 */
const getColorEndPoint = function(startX, startY, origanX, origanY, angle) {
  let endX =
    startX +
    (origanX - startX) * Math.cos((angle * Math.PI) / 180) -
    (origanY - startY) * Math.sin((angle * Math.PI) / 180);
  let endY =
    startY +
    (origanX - startX) * Math.sin((angle * Math.PI) / 180) +
    (origanY - startY) * Math.cos((angle * Math.PI) / 180);
  endX = +endX.toFixed(6);
  endY = +endY.toFixed(6);
  if (endX < 0 || endY < 0) {
    startX += 1;
    startY += 1;
    endX += 1;
    endY += 1;
  }
  let point = {
    x1: startX,
    y1: startY,
    x2: endX,
    y2: endY,
  };
  return point;
};

//#endregion

//#region 获取数据字段

/* 获取绑定的字段 */
const getFieldBindingName = function(fields, fieldName) {
  var tt = fields.find((d) => {
    return d.name.toLowerCase() == fieldName.toLowerCase();
  });
  return tt.columnName;
};

//#endregion

/* 获取通用提示框 */
const getTooltip = function(styles) {
  let tooltip = {};

  //--------------------------是否显示-------------------
  tooltip.show = getStyleValue(styles, 'tooltip');
  if (!tooltip.show) {
    return tooltip;
  }
  //--------------------------偏移-------------------

  let hor = getStyleValue(styles, 'tooltipHorOffset');
  let ver = getStyleValue(styles, 'tooltipVerOffset');
  tooltip.position = function(point, params, dom, rect, size) {
    return [point[0] + hor, point[1] + ver];
  };

  //--------------------------背景颜色和边框颜色-------------------
  tooltip.backgroundColor = getStyleValue(
    styles,
    'tooltipBackgroundColor',
    'color'
  );
  tooltip.borderColor = getStyleValue(styles, 'tooltipBorderColor', 'color');
  tooltip.borderWidth = 1;

  //--------------------------设置文本样式-------------------
  let textStyle = getStyleValue(styles, 'tooltipTextStyle').value[0].items;
  tooltip.textStyle = {};
  tooltip.textStyle.fontFamily = getStyleValue(textStyle, 'textFontfamily');
  tooltip.textStyle.color = getStyleValue(textStyle, 'textForeground', 'color');
  tooltip.textStyle.fontSize = getStyleValue(textStyle, 'textFontSize');
  let bgColor = getStyleValue(textStyle, 'textBackgroundColor', 'color');
  tooltip.textStyle.backgroundColor = bgColor;

  //字体粗细
  let fontWeight = getStyleValue(textStyle, 'textFontWeight');
  let isFontWeight = getStyleValue(textStyle, 'textIsFontWeight');
  if (
    isFontWeight &&
    (fontWeight.toLowerCase() == 'normal' ||
      fontWeight.toLowerCase() == 'lighter')
  ) {
    fontWeight = 'bold';
  }
  tooltip.textStyle.fontWeight = fontWeight;

  //是否倾斜
  tooltip.textStyle.fontStyle = 'normal';
  let fontRotateOnBool = getStyleValue(textStyle, 'textFontRotate');
  if (fontRotateOnBool == true) {
    tooltip.textStyle.fontStyle = 'italic';
  }

  //--------------------------添加阴影，设置宽度-------------------
  let maxWidth = getStyleValue(styles, 'tooltipMaxWidth');
  tooltip.extraCssText = `max-width: ${maxWidth}px;`;
  let isExtraCssText = getStyleValue(styles, 'tooltipIsShowShadow');
  if (isExtraCssText) {
    tooltip.extraCssText = `${tooltip.extraCssText} box-shadow: 0 0 10px rgba(0, 0, 0, 1);`;
  }

  //--------------------------指示器-------------------
  tooltip.axisPointer = {};
  let showPointer = getStyleValue(styles, 'tooltipAxisIndicatorIsShow');
  tooltip.axisPointer.axis = 'auto';
  if (showPointer) {
    let type = getStyleValue(styles, 'tooltipAxisIndicatorType');
    let color = getStyleValue(styles, 'tooltipAxisIndicatorColor', 'color');
    let width = getStyleValue(styles, 'tooltipAxisIndicatorLineWidth');
    if (type == '直线指示器') {
      tooltip.trigger = 'axis';
      tooltip.axisPointer.type = 'line';
      tooltip.axisPointer.lineStyle = {
        color: color,
        width: width,
      };
    } else if (type == '十字指示器') {
      tooltip.axisPointer.type = 'cross';
      tooltip.axisPointer.crossStyle = {
        color: color,
        width: width,
      };
    }
  }

  //--------------------------触发方式-------------------
  tooltip.triggerOn = 'mousemove';
  let triggerMode = getStyleValue(styles, 'tooltipTriggerMode');
  if (triggerMode == '点击') {
    tooltip.triggerOn = 'click';
  }
  return tooltip;
};

/* 获取通用类别轴 */
const getDimensionAxis = function(styles) {
  let xAxis = {
    type: 'category',
    z: 10000,
  };

  //--------------------------是否显示-------------------
  xAxis.show = getStyleValue(styles, 'dimensionAxis');
  if (!xAxis.show) {
    return xAxis;
  }

  //--------------------------轴标题-------------------
  let showTitle = getStyleValue(styles, 'dimensionAxisTitleIsShow');
  if (showTitle) {
    xAxis.name = getStyleValue(styles, 'dimensionAxisTitleName');

    //对齐
    xAxis.nameLocation = 'middle';
    let position = getStyleValue(styles, 'dimensionAxisTitleAlignment');
    if (position == '居左' || position == '居下') {
      xAxis.nameLocation = 'start';
    } else if (position == '居右' || position == '居上') {
      xAxis.nameLocation = 'end';
    }

    //文本样式
    let textStyle = getStyleValue(styles, 'dimensionAxisTitleTextStyle');
    xAxis.nameTextStyle = getTextStyleObj(textStyle.value[0].items);
  }

  //--------------------------轴线-------------------
  let isShowLine = getStyleValue(styles, 'dimensionAxisLineIsShow');
  xAxis.axisLine = { show: isShowLine };
  if (isShowLine) {
    xAxis.axisLine.lineStyle = {};
    xAxis.axisLine.lineStyle.color = getStyleValue(
      styles,
      'dimensionAxisLineColor',
      'color'
    );
    xAxis.axisLine.lineStyle.width = getStyleValue(
      styles,
      'dimensionAxisLineWidth'
    );
  }

  //--------------------------刻度-------------------
  xAxis.axisTick = {};
  let isShowTick = getStyleValue(styles, 'dimensionAxisShowTickLine');
  xAxis.axisTick.show = isShowTick;
  if (isShowTick) {
    xAxis.axisTick.lineStyle = {};
    xAxis.axisTick.lineStyle.color = getStyleValue(
      styles,
      'dimensionAxisLineColor',
      'color'
    );
    xAxis.axisTick.lineStyle.width = getStyleValue(
      styles,
      'dimensionAxisLineWidth'
    );
  }

  //--------------------------分割线-------------------
  xAxis.splitLine = {};
  let isShowSplitLine = getStyleValue(styles, 'dimensionAxisSplitLineIsShow');
  if (isShowSplitLine) {
    let lineStyle = {};
    lineStyle.color = getStyleValue(
      styles,
      'dimensionAxisSplitLineColor',
      'color'
    );
    lineStyle.width = getStyleValue(styles, 'dimensionAxisSplitLineWidth');
    let lineType = getStyleValue(styles, 'dimensionAxisSplitLineType');
    lineStyle.type = 'solid';
    if (lineType == '虚线') {
      lineStyle.type = 'dashed';
    } else if (lineType == '点线') {
      lineStyle.type = 'dotted';
    }

    xAxis.splitLine.show = isShowSplitLine;
    xAxis.splitLine.lineStyle = lineStyle;
  }

  //--------------------------轴标签-------------------
  let isShowLabel = getStyleValue(styles, 'dimensionAxisLabelIsShow');
  xAxis.axisLabel = { show: isShowLabel };
  if (isShowLabel) {
    let textStyle = getStyleValue(styles, 'dimensionAxisLabelTextStyle');
    xAxis.axisLabel = getTextStyleObj(textStyle.value[0].items);

    //是否显示所有标签
    let isShowAll = getStyleValue(styles, 'dimensionAxisLabelShowAllLabel');
    xAxis.axisLabel.interval = isShowAll ? 0 : 'auto';

    //标签角度
    xAxis.axisLabel.rotate = 0;
    let rotate = getStyleValue(styles, 'dimensionAxisLabelRotate');
    if (rotate == '倾斜') {
      xAxis.axisLabel.rotate = 45;
    } else if (rotate == '垂直') {
      xAxis.axisLabel.rotate = -90;
    }

    //文字数量
    let chartCount = getStyleValue(styles, 'dimensionAxisLabelTextMaxCount');
    xAxis.axisLabel.formatter = function(value) {
      if (typeof value == 'string' && value.length > chartCount) {
        return value.substr(0, chartCount);
      }
      return value;
    };

    //偏移
    let margin = getStyleValue(styles, 'dimensionAxisLabelVerOffset');
    xAxis.axisLabel.margin = margin;
  }

  //--------------------------分割区域-------------------
  xAxis.splitArea = {};
  let isShowSplitArea = getStyleValue(styles, 'dimensionAxisGridColorIsShow');
  if (isShowSplitArea) {
    xAxis.splitArea.show = isShowSplitArea;
    xAxis.splitArea.areaStyle = {};
    xAxis.splitArea.areaStyle.color = [
      getStyleValue(styles, 'dimensionAxisGridColor1', 'color'),
      getStyleValue(styles, 'dimensionAxisGridColor2', 'color'),
    ];
  }
  return xAxis;
};

/* 获取通用数量轴 */
const getMeasureAxis = function(styles) {
  let yAxis = { type: 'value' };

  //--------------------------是否显示-------------------
  yAxis.show = getStyleValue(styles, 'measureAxis');
  if (!yAxis.show) {
    return yAxis;
  }

  //--------------------------轴标题-------------------
  let showTitle = getStyleValue(styles, 'measureAxisTitleIsShow');
  if (showTitle) {
    yAxis.name = getStyleValue(styles, 'measureAxisTitleName');

    //对齐
    yAxis.nameLocation = 'middle';
    let position = getStyleValue(styles, 'measureAxisTitleAlignment');
    if (position == '居下' || position == '居左') {
      yAxis.nameLocation = 'start';
    } else if (position == '居上' || position == '居右') {
      yAxis.nameLocation = 'end';
    }

    let textStyle = getStyleValue(styles, 'measureAxisTitleTextStyle');
    yAxis.nameTextStyle = getTextStyleObj(textStyle.value[0].items);
  }

  //--------------------------轴线-------------------
  let isShowLine = getStyleValue(styles, 'measureAxisLineIsShow');
  yAxis.axisLine = { show: isShowLine };
  if (isShowLine) {
    yAxis.axisLine.lineStyle = {};
    yAxis.axisLine.lineStyle.color = getStyleValue(
      styles,
      'measureAxisLineColor',
      'color'
    );
    yAxis.axisLine.lineStyle.width = getStyleValue(
      styles,
      'measureAxisLineWidth'
    );
  }

  //--------------------------刻度-------------------
  let isShowTick = getStyleValue(styles, 'measureAxisShowTickLine');
  yAxis.axisTick = { show: isShowTick };
  if (isShowTick) {
    yAxis.axisTick.lineStyle = {};
    yAxis.axisTick.lineStyle.color = getStyleValue(
      styles,
      'measureAxisLineColor',
      'color'
    );
    yAxis.axisTick.lineStyle.width = getStyleValue(
      styles,
      'measureAxisLineWidth'
    );
  }

  //--------------------------分割线-------------------
  let isShowSplitLine = getStyleValue(styles, 'measureAxisSplitLineIsShow');
  yAxis.splitLine = { show: isShowSplitLine };
  if (isShowSplitLine) {
    let lineStyle = {};
    lineStyle.color = getStyleValue(
      styles,
      'measureAxisSplitLineColor',
      'color'
    );
    lineStyle.width = getStyleValue(styles, 'measureAxisSplitLineWidth');
    let lineType = getStyleValue(styles, 'measureAxisSplitLineType');
    lineStyle.type = 'solid';
    if (lineType == '虚线') {
      lineStyle.type = 'dashed';
    } else if (lineType == '点线') {
      lineStyle.type = 'dotted';
    }

    yAxis.splitLine.lineStyle = lineStyle;
  }

  //--------------------------轴标签-------------------
  let isShowLabel = getStyleValue(styles, 'measureAxisLabelIsShow');
  yAxis.axisLabel = { show: isShowLabel };
  if (isShowLabel) {
    //文本样式
    let textStyle = getStyleValue(styles, 'measureAxisLabelTextStyle');
    yAxis.axisLabel = getTextStyleObj(textStyle.value[0].items);

    //标签文字格式
    let step = getStyleValue(styles, 'globalStyleNumberStep');
    let precision = getStyleValue(styles, 'globalStyleNumberPrecision');
    let showThou = getStyleValue(styles, 'globalStyleNumberShowThousandth');
    yAxis.axisLabel.formatter = function(value) {
      let content = step == 0 ? 0 : value / step;
      if (showThou) {
        content = commonFunction.thousandthPercentile(content, precision);
      } else {
        content = content.toFixed(precision);
      }
      return content;
    };

    //偏移
    yAxis.axisLabel.margin = getStyleValue(styles, 'measureAxisLabelHorOffset');
  }

  //--------------------------分割区域-------------------
  yAxis.splitArea = {};
  let isShowSplitArea = getStyleValue(styles, 'measureAxisGridColorIsShow');
  if (isShowSplitArea) {
    yAxis.splitArea.show = isShowSplitArea;
    yAxis.splitArea.areaStyle = {};
    yAxis.splitArea.areaStyle.color = [
      getStyleValue(styles, 'measureAxisGridColor1', 'color'),
      getStyleValue(styles, 'measureAxisGridColor2', 'color'),
    ];
  }

  //--------------------------数值间隔-------------------
  yAxis.max = getStyleValue(styles, 'measureAxisValueMax');
  yAxis.min = getStyleValue(styles, 'measureAxisValueMin');
  let splitNumber = getStyleValue(styles, 'measureAxisSplitNumber');
  yAxis.splitNumber = (yAxis.max - yAxis.min) / splitNumber;
  return yAxis;
};

/* 获取轴单位样式 */
const getAxisUnitStyle = function(styles) {
  let unitStyle = { position: 'absolute', zIndex: 9999 };

  //是否显示
  let isShow = getStyleValue(styles, 'measureAxisUnitIsShow');
  if (!isShow) {
    unitStyle.display = 'none';
    return unitStyle;
  }

  //文字样式
  let textStyle = getStyleValue(styles, 'measureAxisUnitTextStyle');
  let style = getTextStyleObj(textStyle.value[0].items);
  Object.assign(unitStyle, style);
  unitStyle.fontSize += 'px';

  //偏移
  let hor = getStyleValue(styles, 'measureAxisUnitHorOffset');
  let ver = getStyleValue(styles, 'measureAxisUnitVerOffset');
  unitStyle.marginTop = `${ver}px`;
  unitStyle.marginLeft = `${hor}px`;

  //设置阴影
  let showShader = getStyleValue(styles, 'globalStyleBarIsOpenShadow');
  if (showShader) {
    unitStyle.textShadow = '6px 3px 1px  rgba(0,0,0,1)';
  }

  return unitStyle;
};

/* 获取分页样式 */
const getPagingStyle = function(styles) {
  let paging = {
    location: { position: 'absolute', zIndex: 9999 },
  };
  let isShow = getStyleValue(styles, 'paging');
  if (!isShow) {
    paging.location.display = 'none';
    return paging;
  }

  //高度
  let height = getStyleValue(styles, 'pagingUnitHeight');
  paging.location.height = height + 'px';

  //设置阴影
  let showShader = getStyleValue(styles, 'globalStyleBarIsOpenShadow');
  if (showShader) {
    paging.location.textShadow = '6px 3px 1px  rgba(0,0,0,1)';
  }

  //偏移
  let horOffset = getStyleValue(styles, 'pagingHorOffset');
  let verOffset = getStyleValue(styles, 'pagingVerOffset');

  paging.location.marginTop = verOffset + 'px';

  //对齐方式
  let alignment = getStyleValue(styles, 'pagingAlignment');
  if (alignment == '居左') {
    paging.location.letf = '0%';
    paging.location.marginLeft = horOffset + 'px';
  } else if (alignment == '居右') {
    paging.location.right = '0%';
    paging.location.marginRight = horOffset + 'px';
  } else {
    paging.location.left = '50%';
    paging.location.marginLeft = horOffset + 'px';
    paging.location.transform = 'translate(-50%)';
  }

  //文本间距
  let spacing = getStyleValue(styles, 'pagingUnitSpacing');

  //选中文本
  let checked = getStyleValue(styles, 'pagingSelectedTextStyle');
  paging.checked = getTextStyleObj(checked.value[0].items);
  paging.checked.fontSize += 'px';
  paging.checked.lineHeight = height + 'px';
  paging.checked.margin = [0, spacing + 'px'];
  paging.checked.cursor = 'pointer';

  //未选中
  let unchecked = getStyleValue(styles, 'pagingUnselectedTextStyle');
  paging.unchecked = getTextStyleObj(unchecked.value[0].items);
  paging.unchecked.fontSize += 'px';
  paging.unchecked.lineHeight = height + 'px';
  paging.unchecked.margin = [0, spacing + 'px'];
  paging.unchecked.cursor = 'pointer';

  return paging;
};

/* 获取标题样式 */
const getTitleStyle = function(styles) {
  let titleStyle = {
    location: {
      position: 'absolute',
      zIndex: 9999,
      display: 'flex',
      alignItems: 'center',
    },
    imgStyle: { zIndex: 9999 },
  };

  //是否显示
  let isShow = getStyleValue(styles, 'title');
  if (!isShow) {
    titleStyle.location.display = 'none';
    return titleStyle;
  }

  //对齐方式和偏移
  let alignment = getStyleValue(styles, 'titleAlignment');
  let hor = getStyleValue(styles, 'titleHorOffset');
  let ver = getStyleValue(styles, 'titleVerOffset');

  if (alignment == '居左') {
    titleStyle.location.letf = '0%';
    titleStyle.location.marginLeft = hor + 'px';
  } else if (alignment == '居右') {
    titleStyle.location.right = '0%';
    titleStyle.location.marginRight = hor + 'px';
  } else {
    titleStyle.location.left = '50%';
    titleStyle.location.marginLeft = hor + 'px';
    titleStyle.location.transform = 'translate(-50%)';
  }
  titleStyle.location.marginTop = ver + 'px';

  //设置阴影
  let showShader = getStyleValue(styles, 'globalStyleBarIsOpenShadow');
  if (showShader) {
    titleStyle.location.textShadow = '6px 3px 1px  rgba(0,0,0,1)';
  }

  //文本样式
  let textStyles = getStyleValue(styles, 'titleTextStyle').value[0].items;
  let style = getTextStyleObj(textStyles);
  style.fontSize += 'px';
  Object.assign(titleStyle.location, style);

  //图表样式
  let showIcon = getStyleValue(styles, 'titleIconStyleIsShow');
  if (!showIcon) {
    titleStyle.imgStyle.display = 'none';
  } else {
    let imgver = getStyleValue(styles, 'titleIconVerOffset');
    let imghor = getStyleValue(styles, 'titleIconHorOffset');
    let imgZoom = getStyleValue(styles, 'titleIconZoom');

    titleStyle.imgStyle.transform = `translate(${imghor}px, ${imgver}px) scale(${imgZoom})`;
    titleStyle.imgStyle.filter = 'mask(color = #fff)';
  }

  return titleStyle;
};

/* 设置通用标准线 */
const setMarkLine = function(styles, chart, data) {
  //--------------------------标准线-------------------
  let showLine = getStyleValue(styles, 'measureAxisMarkLineIsShow');
  if (showLine) {
    //标准线文字样式
    let textStyles = getStyleValue(styles, 'measureAxisMarkLineLabelStyle');
    let textStyle = getTextStyleObj(textStyles.value[0].items);
    //标准线样式
    let linesStyle = getStyleValue(styles, 'measureAxisMarkLineStyle');
    for (let lineStyle of linesStyle.value) {
      let style = lineStyle.items;
      let line = {
        name: lineStyle.name,
        lineStyle: {},
      };
      let width = getStyleValue(style, 'markLinWidth');
      line.lineStyle.width = width;
      let color = getStyleValue(style, 'markLinColor', 'color');
      line.lineStyle.color = color;

      let type = getStyleValue(style, 'markLinStyle');
      if (type == '虚线') {
        line.lineStyle.type = 'dashed';
      } else if (type == '点线') {
        line.lineStyle.type = 'dotted';
      } else {
        line.lineStyle.type = 'solid';
      }

      let showLabel = getStyleValue(style, 'markLinIsShowLabel');
      line.label = { show: showLabel };
      if (showLabel) {
        Object.assign(line.label, textStyle);
        line.label.position = 'middle';
        let pos = getStyleValue(style, 'markLinLabelPosition');
        if (pos == '居左') {
          line.label.position = 'insideStartBottom';
        } else if (pos == '居右') {
          line.label.position = 'insideEndBottom';
        }
      }

      let lineType = getStyleValue(style, 'markLinType');
      switch (lineType) {
        case '最大值':
          line.type = 'max';
          break;
        case '最小值':
          line.type = 'min';
          break;
        case '平均值':
          line.type = 'average';
          break;
        case '中线':
          line.type = 'median';
          break;
        case '恒值':
          line.yAxis = getStyleValue(style, 'markLinNumber');
          break;
        case '百分比线':
          // let max = null;
          // let min = null;
          // let measureName = getFieldBindingName(this.fields, 'measure');
          // for (let row of this.rowData) {
          //   let measure = row[measureName];
          //   if (max == null || measure > max) {
          //     max = measure;
          //   } else if (min == null || measure < min) {
          //     min = measure;
          //   }
          // }
          // let number = getStyleValue(style, 'markLinNumber');
          // line.yAxis = min + ((max - min) * number) / 100;
          break;
        case '最大目标值':
          break;
        case '最小目标值':
          break;
      }

      for (let serie of chart.series) {
        if (!serie.markLine.data) {
          serie.markLine = { symbol: 'none', data: [] };
        }
        serie.markLine.data.push(line);
      }
    }
  }
};

/* 设置通用阴影 */
const setShadow = function name(styles, chart) {
  let showShader = getStyleValue(styles, 'globalStyleBarIsOpenShadow');
  if (!showShader) {
    return;
  }
  let shadowColor = 'rgba(0,0,0,1)';
  let shadowOffsetX = 6;
  let shadowOffsetY = 3;
  let shadowBlur = 1;

  // 横轴阴影（轴线阴影和文字阴影）
  // 轴线阴影
  chart.xAxis.axisLine.lineStyle.shadowColor = shadowColor;
  chart.xAxis.axisLine.lineStyle.shadowOffsetX = shadowOffsetX;
  chart.xAxis.axisLine.lineStyle.shadowOffsetY = shadowOffsetY;
  chart.xAxis.axisLine.lineStyle.shadowBlur = shadowBlur;
  // 轴文字阴影
  chart.xAxis.axisLabel.textShadowColor = shadowColor;
  chart.xAxis.axisLabel.textShadowOffsetX = shadowOffsetX;
  chart.xAxis.axisLabel.textShadowOffsetY = shadowOffsetY;
  chart.xAxis.axisLabel.textShadowBlur = shadowBlur;

  // 纵轴阴影（轴线阴影和文字阴影）
  // 轴线阴影
  chart.yAxis.axisLine.lineStyle.shadowColor = shadowColor;
  chart.yAxis.axisLine.lineStyle.shadowOffsetX = shadowOffsetX;
  chart.yAxis.axisLine.lineStyle.shadowOffsetY = shadowOffsetY;
  chart.yAxis.axisLine.lineStyle.shadowBlur = shadowBlur;
  // 轴文字阴影
  chart.yAxis.axisLabel.textShadowColor = shadowColor;
  chart.yAxis.axisLabel.textShadowOffsetX = shadowOffsetX;
  chart.yAxis.axisLabel.textShadowOffsetY = shadowOffsetY;
  chart.yAxis.axisLabel.textShadowBlur = shadowBlur;
  if (chart.legend) {
    chart.legend.textStyle.textShadowColor = shadowColor;
    chart.legend.textStyle.textShadowOffsetX = shadowOffsetX;
    chart.legend.textStyle.textShadowOffsetY = shadowOffsetY;
    chart.legend.textStyle.textShadowBlur = shadowBlur;
  }
};

/* 设置通用动画 */
const setAnimation = function(styles, chart) {
  chart.animation = getStyleValue(styles, 'animation');
  chart.animationDuration = getStyleValue(styles, 'animationDuration');
  chart.animationEasing = getStyleValue(styles, 'animationDurationEffect');

  //--------------------------依次执行-------------------
  let isPolling = getStyleValue(styles, 'animationTypePolling');
  if (isPolling) {
    let interval = chart.animationDuration / 2;
    chart.animationDelay = function(index) {
      return interval * index;
    };
  }

  //--------------------------数据更新动画-------------------
  let isUpdate = getStyleValue(styles, 'animationDataUpdateIsOpen');
  chart.animationDurationUpdate = 0;
  if (isUpdate) {
    chart.animationDurationUpdate = chart.animationDuration;
  }
};

/* 获取文本样式对象 */
const getTextStyleObj = function(textStyle) {
  let style = {};
  style.fontFamily = getStyleValue(textStyle, 'textFontfamily');
  style.color = getStyleValue(textStyle, 'textForeground', 'color');
  style.fontSize = getStyleValue(textStyle, 'textFontSize');
  let bgColor = getStyleValue(textStyle, 'textBackgroundColor', 'color');
  style.backgroundColor = bgColor;

  //字体粗细
  let fontWeight = getStyleValue(textStyle, 'textFontWeight');
  let isFontWeight = getStyleValue(textStyle, 'textIsFontWeight');
  if (
    isFontWeight &&
    (fontWeight.toLowerCase() == 'normal' ||
      fontWeight.toLowerCase() == 'lighter')
  ) {
    fontWeight = 'bold';
  }
  style.fontWeight = fontWeight;

  //是否倾斜
  style.fontStyle = 'normal';
  let fontRotateOnBool = getStyleValue(textStyle, 'textFontRotate');
  if (fontRotateOnBool == true) {
    style.fontStyle = 'italic';
  }

  return style;
};

/* 设置通用图例*/
const getLegend = function(styles) {
  let legend = {};
  //是否显示
  legend.show = getStyleValue(styles, 'legends');
  if (!legend.show) {
    return legend;
  }
  legend.data = [];
  let legendPosition = getStyleValue(styles, 'legendPosition');
  let size = getStyleValue(styles, 'globalStyleSize') * 50;
  let textStyles = getStyleValue(styles, 'legendTextStyle').value[0].items;
  let textStyle = getTextStyleObj(textStyles);
  let chartCount = getStyleValue(styles, 'legendTextMaxCount');
  legend.itemWidth = size;
  legend.itemHeight = size;
  legend.textStyle = textStyle;
  legend.formatter = function(value) {
    if (typeof value == 'string' && value.length > chartCount) {
      return value.substr(0, chartCount);
    }
    return value;
  };
  let position = {};
  let offset = getStyleValue(styles, 'legendOffset');
  let legendWidthOrheight = getStyleValue(styles, 'legendWidthOrheight');
  if (legendPosition == '顶部') {
    position = {
      height: legendWidthOrheight,
      left: offset,
      top: 0,
      orient: 'horizontal',
    };
  } else if (legendPosition == '底部') {
    position = {
      height: legendWidthOrheight,
      bottom: 0,
      left: offset,
      orient: 'horizontal',
    };
  } else if (legendPosition == '左部') {
    position = {
      width: legendWidthOrheight,
      left: 0,
      top: offset,
      orient: 'vertical',
    };
  } else if (legendPosition == '右部') {
    position = {
      width: legendWidthOrheight,
      right: 0,
      top: offset,
      orient: 'vertical',
    };
  }
  legend = Object.assign(legend, position);
  return legend;
};

export default {
  getStyleValue,
  getTooltip,
  getDimensionAxis,
  getMeasureAxis,
  setAnimation,
  setShadow,
  getFieldBindingName,
  getTextStyleObj,
  getColor,
  setMarkLine,
  getAxisUnitStyle,
  getPagingStyle,
  getTitleStyle,
  getLegend,
  ...commonFunction,
};
