#!/usr/bin/env node

'use strict';

const __root = process.cwd();
const fs = require('fs');
const path = require('path');
const program = require('commander');
const pjson = require('./package.json');

const PROJECT_TEMPLATE_PATH = path.resolve(__dirname, './.templates/project');
const COMPONENT_TEMPLATE_PATH = path.resolve(__dirname, './.templates/component');
const COMPONENT_DESC_PATH = path.resolve(__dirname, './.templates/description');
const COMPONENT_IMG_PATH = path.resolve(__dirname, './.templates/images');
const SCENE_LAYER_PATH = path.resolve(__dirname, './.templates/scene-layer');

const COMPONENT_NAME_CATEGORY = '__component_category__';
const COMPONENT_NAME_TEMPLATE = '__component__';
const COMPONENT_NAME_TEMPLATE_CAMEL = '__component_camel__';
const COMPONENT_NAME_TEMPLATE_HYPHEN = '__component_hyphen__';
const COMPONENT_NAME_TEMPLATE_CHINESE = '__component_chinese__';
const COMPONENT_NAME_TEMPLATE_LOWER_CASE = '__component_lower_case__';

const JS_IMPORT = '控件导入开始';
const JS_DEFINE = 'const components = \\[';
const JS_TYPES = 'chartTypeDictionary = \\{';
const JS_DATA_PATH = 'const componentDescribeDataPath = \\[';

const SCENE_IMPORT = '多场景地图图层导入开始';
const SCENE_DATA_PATH = 'const scenelayerDescribeDatapath = \\[';
const SCENE_TYPES = 'const layerJsDictionary = \\{';

var componentCategory = '';
var componentPascalName = '';
var componentCamelName = '';
var componentHyphenName = '';
var componentChineseName = '';

var layerPascalName = '';
var layerCamelName = '';
var layerHyphenName = '';
var layerChineseName = '';

function initProj(projectName) {
  if (!projectName) {
    return;
  }
  console.log(`准备创建 AVW 二次开发组件项目：${projectName}...`);
  var projectPath = __root + '/' + projectName;
  if (!fs.existsSync(projectPath)) {
    fs.mkdirSync(projectPath);
    copyFolder(PROJECT_TEMPLATE_PATH, projectPath);
    console.log('信息: 已创建 AVW 二次开发组件项目。');
    // 创建成功以后，进入项目目录
    console.log('信息: 正在复制文件...');
    setTimeout(function () {
      console.log('信息: 项目创建成功，您还需要进入项目目录并运行 "npm install" 完成项目设置。');
      console.log(
        `
===================================   提示   ===================================
要启动 SDK，请进入项目目录，先运行 "npm install"，然后运行 "npm run build"，最后运行 "npm run serve"。
================================================================================
`
      );
    }, 3000);
  } else {
    console.error(`错误: ${projectName} 项目已经存在，请使用其他名称。`);
  }
}

// 创建新组件
function createComponent(componentStr) {
  if (!componentStr) {
    return;
  }
  let componentNames = componentStr.toString().split('/');
  for (let i = 0; i < componentNames.length; i++) {
    componentNames[i] = componentNames[i].replace(/ /g, '');
  }
  if (!componentNames || !componentNames.length || componentNames.length != 2) {
    console.error('错误: 请输入创建组件的英文名称、组件中文名称，用 "/" 连接，如 "avwsdk -c MyComponent/我的组件"。');
    return;
  }
  if (!/^[A-Z][A-Za-z0-9]+$/.test(componentNames[0])) {
    console.error('错误: 组件英文名称请使用 Pascal 命名法（每个单词首字母大写，单词与单词之间不加空格或连接符，如 "MyComponent"）。');
    return;
  }
  if (!/^[\u4e00-\u9fa5A-Za-z0-9]+$/.test(componentNames[1])) {
    console.error('错误: 组件中文名称请使用中文。');
    return;
  }
  var componentPath = `${__root}/src/charts/${pascalToHyphenName(componentNames[0])}`;
  if (!fs.existsSync(`${__root}/package.json`)) {
    console.error('错误: 请先创建项目（avwsdk -p <projectName>），然后确保在项目根目录中运行 avwsdk -c <name>/<chinese> 命令。');
    return;
  }
  if (!fs.existsSync(componentPath)) {
    if (!/^[\u4e00-\u9fa5][\u4e00-\u9fa5A-Za-z0-9]+$/.test(componentNames[1])) {
      console.error('警告: 组件中文名称未使用中文。出于用户体验原因，建议使用中文。这是非强制性的建议，您可以继续开发。');
    }
    try {
      fs.mkdirSync(componentPath);

      componentPascalName = componentNames[0];
      componentCamelName = pascalToCamelName(componentPascalName);
      componentHyphenName = pascalToHyphenName(componentPascalName);
      componentChineseName = componentNames[1];
      copyFolder(COMPONENT_TEMPLATE_PATH, componentPath);

      var componentDescPath = componentPath;
      var componentImgPath = componentPath;
      // 如果不存在这个目录，创建之
      if (!fs.existsSync(componentDescPath)) {
        fs.mkdirSync(componentDescPath);
      }
      if (!fs.existsSync(componentImgPath)) {
        fs.mkdirSync(componentImgPath);
      }
      copyFolder(COMPONENT_DESC_PATH, componentDescPath);
      copyFolder(COMPONENT_IMG_PATH, componentImgPath);
      // 设置项目根层级的 data.js
      // 设置导入
      replaceFileSync(`${__root}/src/charts/data.js`, JS_IMPORT, `${JS_IMPORT}\nimport ${componentPascalName} from './${componentHyphenName}/index';`);
      // 设置 components 数组
      replaceFileSync(`${__root}/src/charts/data.js`, JS_DEFINE, `${JS_DEFINE.replace('\\[', '[')}\n    ${componentPascalName},`);
      // 设置 chartTypeDictionary 对象
      replaceFileSync(
        `${__root}/src/charts/data.js`,
        JS_TYPES,
        `${JS_TYPES.replace('\\{', '{')}\n    ${componentPascalName}: './${componentHyphenName}/index',`
      );
      // 设置描述文件
      replaceFileSync(
        `${__root}/src/charts/data.js`,
        JS_DATA_PATH,
        `${JS_DATA_PATH.replace('\\[', '[')}\n    './lib/avw-charts/assets/${componentHyphenName}/${componentHyphenName}.json',`
      );
      console.warn(`信息: 新控件创建成功。`);
    } catch (error) {
      console.error('错误：路径错误，请确保当前路径为项目的根目录。');
    }
  } else {
    // 已经存在目录
    console.warn(`错误: ${componentNames[0]} 已经存在，请使用其他名称。`);
    return;
  }
}

// 创建新组件
function createSceneLayer(layerStr) {
  if (!layerStr) {
    return;
  }
  let layerNames = layerStr.toString().split('/');
  for (let i = 0; i < layerNames.length; i++) {
    layerNames[i] = layerNames[i].replace(/ /g, '');
  }
  if (!layerNames || !layerNames.length || layerNames.length != 2) {
    console.error('错误: 请输入创建多场景地图图层英文名称、组件中文名称，用 "/" 连接，如 "avwsdk -s MyLayer/我的图层"。');
    return;
  }
  if (!/^[A-Z][A-Za-z0-9]+$/.test(layerNames[0])) {
    console.error('错误: 多场景地图图层英文名称请使用 Pascal 命名法（每个单词首字母大写，单词与单词之间不加空格或连接符，如 "MyLayer"）。');
    return;
  }
  if (!/^[\u4e00-\u9fa5A-Za-z0-9]+$/.test(layerNames[1])) {
    console.error('错误: 多场景地图图层中文名称请使用中文。');
    return;
  }
  var layerCategoryPath = ``;
  var layerPath = `${__root}/src/charts/scene-${pascalToHyphenName(layerNames[0])}`;
  if (!fs.existsSync(`${__root}/package.json`)) {
    console.error('错误: 请先创建项目（avwsdk -p <projectName>），然后确保在项目根目录中运行 avwsdk -s <name>/<chinese> 命令。');
    return;
  }
  if (!fs.existsSync(layerPath)) {
    if (!/^[\u4e00-\u9fa5][\u4e00-\u9fa5A-Za-z0-9]+$/.test(layerNames[1])) {
      console.error('警告: 多场景地图图层中文名称未使用中文。出于用户体验原因，建议使用中文。这是非强制性的建议，您可以继续开发。');
    }
    try {
      fs.mkdirSync(layerPath);
      layerPascalName = layerNames[0];
      layerCamelName = pascalToCamelName(layerPascalName);
      layerHyphenName = pascalToHyphenName(layerPascalName);
      layerChineseName = layerNames[1];
      copyFolder(SCENE_LAYER_PATH, layerPath);

      const SCENE_IMPORT = '多场景地图图层导入开始';
      const SCENE_DATA_PATH = 'const scenelayerDescribeDatapath = \\[';
      const SCENE_TYPES = 'const layerJsDictionary = \\{';

      // 设置项目根层级的 data.js
      // 设置导入
      replaceFileSync(
        `${__root}/src/charts/data.js`,
        SCENE_IMPORT,
        `${SCENE_IMPORT}\nimport Scene${layerPascalName} from './scene-${layerHyphenName}/scene-${layerHyphenName}.js';`
      );
      // 设置 components 数组
      replaceFileSync(
        `${__root}/src/charts/data.js`,
        SCENE_TYPES,
        `${SCENE_TYPES.replace('\\{', '{')}\n    'scene-${layerHyphenName}': Scene${layerPascalName},`
      );
      // 设置描述文件
      replaceFileSync(
        `${__root}/src/charts/data.js`,
        SCENE_DATA_PATH,
        `${SCENE_DATA_PATH.replace('\\[', '[')} './lib/avw-charts/assets/scene-${layerHyphenName}/scene-${layerHyphenName}.json', `
      );
      console.warn(`信息: 新的多场景地图图层创建成功。`);
    } catch (error) {
      console.error('错误：路径错误，请确保当前路径为项目的根目录。');
    }
  } else {
    // 已经存在目录
    console.warn(`错误: ${layerNames[0]} 已经存在，请使用其他名称。`);
    return;
  }
}

function copyFolder(src, dst, type) {
  if (!type) {
    type = 'control';
  }
  let paths = fs.readdirSync(src); //同步读取当前目录
  paths.forEach(function (path) {
    var _src = src + '/' + path;
    var _dst = dst + '/' + path;
    fs.stat(_src, function (err, stats) {
      //stats  该对象包含文件属性
      if (err) throw err;
      if (stats.isFile()) {
        //如果是文件则拷贝
        if (path !== '.DS_Store') {
          // 忽略 .DS_Store
          if (_dst.endsWith('component.vue')) {
            _dst = _dst.replace(/component\.vue$/, `${componentPascalName}.vue`);
          }
          if (_dst.endsWith('component-big.png')) {
            _dst = _dst.replace(/component-big\.png$/, `${componentHyphenName}-big.png`);
          }
          if (_dst.endsWith('component-small.png')) {
            _dst = _dst.replace(/component-small\.png$/, `${componentHyphenName}-small.png`);
          }
          if (_dst.endsWith('component.json')) {
            _dst = _dst.replace(/component\.json$/, `${componentHyphenName}.json`);
          }
          if (_dst.endsWith('scene-layer.js')) {
            _dst = _dst.replace(/scene-layer\.js$/, `scene-${layerHyphenName}.js`);
          }
          if (_dst.endsWith('scene-layer-big.png')) {
            _dst = _dst.replace(/scene-layer-big\.png$/, `scene-${layerHyphenName}-big.png`);
          }
          if (_dst.endsWith('scene-layer-small.png')) {
            _dst = _dst.replace(/scene-layer-small\.png$/, `scene-${layerHyphenName}-small.png`);
          }
          if (_dst.endsWith('scene-layer.json')) {
            _dst = _dst.replace(/scene-layer\.json$/, `scene-${layerHyphenName}.json`);
          }
          if (_dst.endsWith('package.template.json')) {
            _dst = _dst.replace(/package\.template\.json$/, `package.json`);
          }
          let readable = fs.createReadStream(_src); //创建读取流
          let writable = fs.createWriteStream(_dst); //创建写入流
          readable.on('end', () => {
            console.log(`信息: 已复制 ${_dst}`);
            switch (path.toLowerCase()) {
              case 'index.js':
                // 修改控件导入入口文件模板中的组件名称
                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE, componentPascalName);
                break;
              case 'component.vue':
                // 修改控件 vue 文件模板中的组件名称
                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_CAMEL, componentCamelName);
                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_CHINESE, componentChineseName);
                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_HYPHEN, componentHyphenName);
                break;
              case 'component.json':
                // 修改控件描述文件模板中的组件名称和中文名称
                replaceFileSync(_dst, COMPONENT_NAME_CATEGORY, componentCategory);
                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_CHINESE, componentChineseName);
                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_HYPHEN, componentHyphenName);
                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_LOWER_CASE, componentPascalName.toLowerCase());
                break;
              case 'scene-layer.js':
                // 修改多场景地图图层文件模板中的名称
                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_CHINESE, layerChineseName);
                break;
              case 'scene-layer.json':
                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_CHINESE, layerChineseName);
                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_HYPHEN, layerHyphenName);
                break;
              default:
                break;
            }
          });
          readable.pipe(writable);
        }
      } else if (stats.isDirectory()) {
        //是目录则递归
        checkDirectory(_src, _dst, copyFolder);
      }
    });
  });
}

function checkDirectory(src, dst, callback) {
  fs.access(dst, fs.constants.F_OK, (err) => {
    if (err) {
      fs.mkdirSync(dst);
      callback(src, dst);
    } else {
      callback(src, dst);
    }
  });
}

function replaceFileSync(filePath, sourceRegx, targetStr) {
  let data = fs.readFileSync(filePath);
  let str = data.toString();
  str = str.replace(new RegExp(sourceRegx, 'gm'), targetStr);
  fs.writeFileSync(filePath, str);
}

function pascalToCamelName(name) {
  if (!name || name == '') {
    return 'invalidName';
  }
  return name.charAt(0).toString().toLowerCase() + name.substring(1, name.length);
}

function camelToHyphenName(name) {
  if (!name || name == '') {
    return 'invalid-name';
  }
  let newName = '';
  for (let i = 0; i < name.length; i++) {
    if (/[A-Z]/.test(name[i])) {
      newName += '-';
    }
    newName += name[i];
  }
  return newName.toLowerCase();
}

function pascalToHyphenName(name) {
  return camelToHyphenName(pascalToCamelName(name));
}

program
  .usage('[options] <names ...>')
  .option('-p, --project <name>', '创建新的 AVW 二次开发项目及其目录')
  .option(
    '-c, --component <name>/<chinese>',
    '创建新控件，请输入要创建的控件英文名称、中文名称，中间用 "/" 分隔，英文名称请使用 Pascal 命名法（如 MyComponent）'
  )
  .option(
    '-s, --scenelayer <name>/<chinese>',
    '创建用于多场景地图的新图层，请输入要创建的图层英文名称、中文名称，中间用 "/" 分隔，英文名称请使用 Pascal 命名法（如 MyLayer）'
  )
  .version(pjson.version, '-v, --version', '显示版本')
  .helpOption('-h, --help', '显示本帮助信息')
  .parse(process.argv);

if (!program.project && !program.component && !program.scenelayer) {
  console.log(`AVW SDK: ${pjson.version}，要显示帮助，请运行 avwsdk -h`);
} else {
  initProj(program.project);
  createComponent(program.component);
  createSceneLayer(program.scenelayer);
}
